@@ -30,9 +30,10 @@
 #include <fcntl.h>
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-moniker-util.h>
+#include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
 #include <libgnomevfs/gnome-vfs.h>
-#include <libedataserver/e-util.h>
+#include <libedataserver/e-data-server-util.h>
 #include <libedataserver/e-xml-hash-utils.h>
 #include <libecal/e-cal-recur.h>
 #include <libecal/e-cal-time-util.h>
@@ -41,6 +42,14 @@
 #include <libedata-cal/e-cal-backend-sexp.h>
 #include "e-cal-backend-file-events.h"
 
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
+#if !GLIB_CHECK_VERSION (2, 8, 0)
+#define g_access access
+#endif
+
 
 
 /* Placeholder for each component and its recurrences */
@@ -616,7 +625,7 @@ get_uri_string_for_gnome_vfs (ECalBackend *backend)
 		return NULL;
 	}
 	
-	full_uri = g_strdup_printf ("%s%s%s", master_uri, G_DIR_SEPARATOR_S, priv->file_name);
+	full_uri = g_strdup_printf ("%s/%s", master_uri, priv->file_name);
 	uri = gnome_vfs_uri_new (full_uri);
 	g_free (full_uri);
 	
@@ -885,9 +894,9 @@ e_cal_backend_file_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_
 	if (!str_uri)
 		return GNOME_Evolution_Calendar_OtherError;
 	
-	if (access (str_uri, R_OK) == 0) {
+	if (g_access (str_uri, R_OK) == 0) {
 		status = open_cal (cbfile, str_uri);
-		if (access (str_uri, W_OK) != 0)
+		if (g_access (str_uri, W_OK) != 0)
 			priv->read_only = TRUE;
 	} else {
 		if (only_if_exists)
@@ -922,7 +931,7 @@ e_cal_backend_file_remove (ECalBackendSync *backend, EDataCal *cal)
 	if (!str_uri)
 		return GNOME_Evolution_Calendar_OtherError;
 
-	if (access (str_uri, W_OK) != 0) {
+	if (g_access (str_uri, W_OK) != 0) {
 		g_free (str_uri);
 
 		return GNOME_Evolution_Calendar_PermissionDenied;
@@ -942,7 +951,7 @@ e_cal_backend_file_remove (ECalBackendSync *backend, EDataCal *cal)
 		char *full_path;
 
 		full_path = g_build_filename (dirname, fname, NULL);
-		if (unlink (full_path) != 0) {
+		if (g_unlink (full_path) != 0) {
 			g_free (full_path);
 			g_free (str_uri);
 			g_free (dirname);
@@ -955,7 +964,7 @@ e_cal_backend_file_remove (ECalBackendSync *backend, EDataCal *cal)
 	}
 
 	/* remove the directory itself */
-	success = rmdir (dirname) == 0;
+	success = g_rmdir (dirname) == 0;
 		
 	g_dir_close (dir);
 	g_free (str_uri);
@@ -1183,8 +1192,6 @@ e_cal_backend_file_set_default_timezone (ECalBackendSync *backend, EDataCal *cal
 	cbfile = E_CAL_BACKEND_FILE (backend);
 	priv = cbfile->priv;
 
-	g_return_val_if_fail (priv->icalcomp != NULL, GNOME_Evolution_Calendar_NoSuchCal);
-
 	/* Set the default timezone to it. */
 	priv->default_zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
 
@@ -1801,7 +1808,7 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 	ECalBackendFile *cbfile;
 	ECalBackendFilePrivate *priv;
 	icalcomponent *icalcomp;
-	const char *comp_uid, *rid;
+	const char *comp_uid, *rid = NULL;
 	char *real_rid;
 	ECalComponent *comp, *recurrence;
 	ECalBackendFileObject *obj_data;
@@ -1843,11 +1850,11 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 
 	/* sanitize the component*/
 	sanitize_component (cbfile, comp);
+	rid = e_cal_component_get_recurid_as_string (comp);
 
 	/* handle mod_type */
 	switch (mod) {
 	case CALOBJ_MOD_THIS :
-		rid = e_cal_component_get_recurid_as_string (comp);
 		if (!rid || !*rid) {
 			if (old_object)
 				*old_object = e_cal_component_get_as_string (obj_data->full_object);
@@ -1880,26 +1887,11 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 			priv->comp = g_list_remove (priv->comp, recurrence);
 			obj_data->recurrences_list = g_list_remove (obj_data->recurrences_list, recurrence);
 			g_hash_table_remove (obj_data->recurrences, rid);
-		} else {
-			char *old, *new;
-
-			old = e_cal_component_get_as_string (obj_data->full_object);
-
-			new = e_cal_component_get_as_string (comp);
-			e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old, new);
-
-			if (old_object)
-				*old_object = old;
-			else
-				g_free (old);
-			g_free (new);
 		}
 
 		/* add the detached instance */
-		icalcomponent_set_recurrenceid (e_cal_component_get_icalcomponent (comp),
-						icaltime_from_string (rid));
 		g_hash_table_insert (obj_data->recurrences, 
-				     g_strdup (e_cal_component_get_recurid_as_string (comp)),
+				     g_strdup (rid),
 				     comp);
 		icalcomponent_add_component (priv->icalcomp,
 					     e_cal_component_get_icalcomponent (comp));
@@ -1967,6 +1959,44 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 	case CALOBJ_MOD_ALL :
 		/* in this case, we blow away all recurrences, and start over
 		   with a clean component */
+
+		if (e_cal_util_component_has_recurrences (icalcomp) && rid && *rid) {
+			icaltimetype start, recur = icaltime_from_string (rid);
+
+			start = icalcomponent_get_dtstart (icalcomp);
+
+			/* This means its a instance generated from master object. So replace 
+			    the dates stored dates from the master object */
+			if (!recur.zone)   
+				recur.zone = start.zone;
+			
+			if (icaltime_compare_date_only (start, recur) == 0) {
+				ECalComponentDateTime m_sdate, m_endate;
+
+				e_cal_component_get_dtstart (obj_data->full_object, &m_sdate);
+				e_cal_component_get_dtend (obj_data->full_object, &m_endate);
+
+				if (icaltime_compare (start, recur) != 0) {
+					icaltimetype end = icalcomponent_get_dtend (icalcomp);
+					
+					m_sdate.value->hour = start.hour;
+					m_sdate.value->minute = start.minute;
+					m_sdate.value->second = 
+						
+					m_endate.value->hour = end.hour;
+					m_endate.value->minute = end.minute;
+					m_endate.value->second = end.second;
+				}
+
+				e_cal_component_set_dtstart (comp, &m_sdate);
+				e_cal_component_set_dtend (comp, &m_endate);
+				e_cal_component_set_recurid (comp, NULL);
+				e_cal_component_commit_sequence (comp);
+			} 
+			e_cal_component_set_recurid (comp, NULL);
+			*new_object = e_cal_component_get_as_string (comp);
+		}
+		
 		/* Remove the old version */
 		if (old_object)
 			*old_object = e_cal_component_get_as_string (obj_data->full_object);
@@ -1999,8 +2029,6 @@ remove_instance (ECalBackendFile *cbfile, ECalBackendFileObject *obj_data, const
 		cbfile->priv->comp = g_list_remove (cbfile->priv->comp, comp);
 		obj_data->recurrences_list = g_list_remove (obj_data->recurrences_list, comp);
 		g_hash_table_remove (obj_data->recurrences, rid);
-
-		return;
 	}
 
 	/* remove the component from our data, temporarily */
@@ -2019,6 +2047,27 @@ remove_instance (ECalBackendFile *cbfile, ECalBackendFileObject *obj_data, const
 	cbfile->priv->comp = g_list_prepend (cbfile->priv->comp, obj_data->full_object);
 }
 
+static char *
+get_object_string_from_fileobject (ECalBackendFileObject *obj_data, const char *rid)
+{
+	ECalComponent *comp = obj_data->full_object;
+	char *real_rid;
+
+	if (!rid) {
+		return e_cal_component_get_as_string (comp);
+	} else {
+		if (g_hash_table_lookup_extended (obj_data->recurrences, rid, (void **)&real_rid, (void **)&comp))
+			return e_cal_component_get_as_string (comp);
+		else {
+			/* FIXME remove this once we delete an instance from master object through
+			   modify request by setting exception */
+			return e_cal_component_get_as_string (comp);
+		}	
+	}
+
+	return NULL;
+}
+
 /* Remove_object handler for the file backend */
 static ECalBackendSyncStatus
 e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
@@ -2031,6 +2080,7 @@ e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
 	ECalBackendFileObject *obj_data;
 	ECalComponent *comp;
 	RemoveRecurrenceData rrdata;
+	const char *recur_id = NULL;
 
 	cbfile = E_CAL_BACKEND_FILE (backend);
 	priv = cbfile->priv;
@@ -2043,37 +2093,35 @@ e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
 	obj_data = g_hash_table_lookup (priv->comp_uid_hash, uid);
 	if (!obj_data)
 		return GNOME_Evolution_Calendar_ObjectNotFound;
-
+	
+	if (rid && *rid)
+		recur_id = rid;
+	
 	comp = obj_data->full_object;
 
 	switch (mod) {
 	case CALOBJ_MOD_ALL :
-		if (comp) {
-			*old_object = e_cal_component_get_as_string (comp);
-		} else {
-			char *real_rid;
-
-			if (g_hash_table_lookup_extended (obj_data->recurrences, rid, (void **)&real_rid, (void **)&comp))
-				*old_object = e_cal_component_get_as_string (comp);
-		}
-
+		*old_object = get_object_string_from_fileobject (obj_data, recur_id);
 		remove_component (cbfile, uid, obj_data);
 
 		*object = NULL;
 		break;
 	case CALOBJ_MOD_THIS :
-		*old_object = e_cal_component_get_as_string (comp);
-		if (!rid || !*rid) {
+		if (!recur_id) {
+			*old_object = get_object_string_from_fileobject (obj_data, recur_id);
 			remove_component (cbfile, uid, obj_data);
 			*object = NULL;
 		} else {
-			remove_instance (cbfile, obj_data, rid);
-			*object = e_cal_component_get_as_string (obj_data->full_object);
+			*old_object = get_object_string_from_fileobject (obj_data, recur_id);
+
+			remove_instance (cbfile, obj_data, recur_id);
+			if (comp)
+				*object = e_cal_component_get_as_string (comp);
 		}
 		break;
 	case CALOBJ_MOD_THISANDPRIOR :
 	case CALOBJ_MOD_THISANDFUTURE :
-		if (!rid || !*rid)
+		if (!recur_id || !*recur_id)
 			return GNOME_Evolution_Calendar_ObjectNotFound;
 
 		*old_object = e_cal_component_get_as_string (comp);
@@ -2084,12 +2132,12 @@ e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
 		priv->comp = g_list_remove (priv->comp, comp);
 
 		e_cal_util_remove_instances (e_cal_component_get_icalcomponent (comp),
-					     icaltime_from_string (rid), mod);
+					     icaltime_from_string (recur_id), mod);
 
 		/* now remove all detached instances */
 		rrdata.cbfile = cbfile;
 		rrdata.obj_data = obj_data;
-		rrdata.rid = rid;
+		rrdata.rid = recur_id;
 		rrdata.mod = mod;
 		g_hash_table_foreach_remove (obj_data->recurrences, (GHRFunc) remove_object_instance_cb, &rrdata);
 
@@ -2155,75 +2203,82 @@ check_tzids (icalparameter *param, void *data)
 		tzdata->found = FALSE;
 }
 
+
+/* This function is largely duplicated in
+ * ../groupwise/e-cal-backend-groupwise.c
+ */
 static void
 fetch_attachments (ECalBackendSync *backend, ECalComponent *comp)
 {
 	GSList *attach_list = NULL, *new_attach_list = NULL;
 	GSList *l;
-	char  *attach_store, *filename, *file_contents;
+	char  *attach_store;
 	char *dest_url, *dest_file;
-	int fd, len;
-	int len_read = 0;
-	char buf[1024];
-	struct stat sb;
+	int fd;
 	const char *uid;
 
-
 	e_cal_component_get_attachment_list (comp, &attach_list);
 	e_cal_component_get_uid (comp, &uid);
 	/*FIXME  get the uri rather than computing the path */
-	attach_store = g_strconcat (g_get_home_dir (), "/",
+	attach_store = g_build_filename (g_get_home_dir (),
 			".evolution/calendar/local/system", NULL);
 	
 	for (l = attach_list; l ; l = l->next) {
 		char *sfname = (char *)l->data;
+		char *filename, *new_filename;
+#if GLIB_CHECK_VERSION (2, 8, 0)
+		GMappedFile *mapped_file;
+#else
+		char *file_contents;
+		int len;
+#endif
+		GError *error = NULL;
 
-		filename = g_strrstr (sfname, "/") + 1;	
-
-		// open the file using the data
-		fd = open (sfname, O_RDONLY); 
-		if (fd == -1) {
-			/* TODO handle error conditions */
-			g_message ("DEBUG: could not open the file descriptor\n");
+#if GLIB_CHECK_VERSION (2, 8, 0)
+		mapped_file = g_mapped_file_new (sfname, FALSE, &error);
+		if (!mapped_file) {
+			g_message ("DEBUG: could not map %s: %s\n",
+				   sfname, error->message);
+			g_error_free (error);
 			continue;
 		}
-		if (fstat (fd, &sb) == -1) {
-			/* TODO handle error conditions */
-			g_message ("DEBUG: could not fstat the attachment file\n");
+#else
+		if (!g_file_get_contents (sfname, &file_contents, &len, &error)) {
+			g_message ("DEBUG: could not read %s: %s\n",
+				   sfname, error->message);
+			g_error_free (error);
 			continue;
 		}
-		len = sb.st_size;
-
-		file_contents = g_malloc (len + 1);
-	
-		while (len_read < len) {
-			int c = read (fd, buf, sizeof (buf));
-
-			if (c == -1)
-				break;
-
-			memcpy (&file_contents[len_read], buf, c);
-			len_read += c;
-		}
-		file_contents [len_read] = 0;
-
-		/* write*/
-		dest_file = g_strconcat (attach_store, "/", uid, "-",
-				filename, NULL);
-		fd = open (dest_file, O_RDWR|O_CREAT|O_TRUNC, 0600);
+#endif
+		filename = g_path_get_basename (sfname);
+		new_filename = g_strconcat (uid, "-", filename, NULL);
+		g_free (filename);
+		dest_file = g_build_filename (attach_store, new_filename, NULL);
+		g_free (new_filename);
+		fd = g_open (dest_file, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, 0600);
 		if (fd == -1) {
 			/* TODO handle error conditions */
-			g_message ("DEBUG: could not serialize attachments\n");
-		}
-
-		if (write (fd, file_contents, len_read) == -1) {
+			g_message ("DEBUG: could not open %s for writing\n",
+				   dest_file);
+#if GLIB_CHECK_VERSION (2, 8, 0)
+		} else if (write (fd, g_mapped_file_get_contents (mapped_file),
+				  g_mapped_file_get_length (mapped_file)) == -1) {
+#else
+		} else if (write (fd, file_contents, len) == -1) {
+#endif
 			/* TODO handle error condition */
 			g_message ("DEBUG: attachment write failed.\n");
 		}
 
-		dest_url = g_strconcat ("file:///", dest_file, NULL);
-		new_attach_list = g_slist_append (new_attach_list, dest_url);
+#if GLIB_CHECK_VERSION (2, 8, 0)
+		g_mapped_file_free (mapped_file);
+#else
+		g_free (file_contents);
+#endif
+		close (fd);
+		dest_url = g_filename_to_uri (dest_file, NULL, NULL);
 		g_free (dest_file);
+		new_attach_list = g_slist_append (new_attach_list, dest_url);
 	}
 	g_free (attach_store);
 	e_cal_component_set_attachment_list (comp, new_attach_list);
@@ -2313,8 +2368,18 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 		}
 		
 		if (!icalcomponent_get_uid (subcomp)) {
-			status = GNOME_Evolution_Calendar_InvalidObject;
-			goto error;
+			if (toplevel_method == ICAL_METHOD_PUBLISH) {
+
+				char *new_uid = NULL;
+
+				new_uid = e_cal_component_gen_uid ();
+				icalcomponent_set_uid (subcomp, new_uid);
+				g_free (new_uid);
+			} else {
+				status = GNOME_Evolution_Calendar_InvalidObject;
+				goto error;
+			}
+			
 		}
 		
 		comps = g_list_prepend (comps, subcomp);
@@ -2599,9 +2664,9 @@ e_cal_backend_file_reload (ECalBackendFile *cbfile)
 	if (!str_uri)
 		return GNOME_Evolution_Calendar_OtherError;
 
-	if (access (str_uri, R_OK) == 0) {
+	if (g_access (str_uri, R_OK) == 0) {
 		status = reload_cal (cbfile, str_uri);
-		if (access (str_uri, W_OK) != 0)
+		if (g_access (str_uri, W_OK) != 0)
 			priv->read_only = TRUE;
 	} else {
 		status = GNOME_Evolution_Calendar_NoSuchCal;
