@@ -31,6 +31,7 @@
 #include <libedataserver/e-xml-hash-utils.h>
 #include <libecal/e-cal-recur.h>
 #include <libecal/e-cal-util.h>
+#include <libecal/e-cal-time-util.h>
 #include <libedata-cal/e-cal-backend-cache.h>
 #include <libedata-cal/e-cal-backend-util.h>
 #include <libedata-cal/e-cal-backend-sexp.h>
@@ -128,6 +129,11 @@ e_cal_backend_http_finalize (GObject *object)
 		priv->soup_session = NULL;
 	}
 
+	if (priv->reload_timeout_id) {
+		g_source_remove (priv->reload_timeout_id);
+		priv->reload_timeout_id = 0;
+	}
+
 	g_free (priv);
 	cbhttp->priv = NULL;
 
@@ -188,23 +194,29 @@ e_cal_backend_http_get_static_capabilities (ECalBackendSync *backend, EDataCal *
 }
 
 static gchar *
-webcal_to_http_method (const gchar *webcal_str)
+webcal_to_http_method (const gchar *webcal_str, gboolean secure)
 {
+	if (secure && (strncmp ("http://", webcal_str, sizeof ("http://") - 1) == 0))
+		return g_strconcat ("https://", webcal_str + sizeof ("http://") - 1, NULL);
+
 	if (strncmp ("webcal://", webcal_str, sizeof ("webcal://") - 1))
 		return g_strdup (webcal_str);
 
-	return g_strconcat ("http://", webcal_str + sizeof ("webcal://") - 1, NULL);
+	if (secure)
+		return g_strconcat ("https://", webcal_str + sizeof ("webcal://") - 1, NULL);
+	else
+		return g_strconcat ("http://", webcal_str + sizeof ("webcal://") - 1, NULL);
 }
 
 static gboolean
 notify_and_remove_from_cache (gpointer key, gpointer value, gpointer user_data)
 {
-	const char *uid = key;
 	const char *calobj = value;
 	ECalBackendHttp *cbhttp = E_CAL_BACKEND_HTTP (user_data);
 	ECalComponent *comp = e_cal_component_new_from_string (calobj);
 	ECalComponentId *id = e_cal_component_get_id (comp);
 
+	e_cal_backend_cache_remove_component (cbhttp->priv->cache, id->uid, id->rid);
 	e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbhttp), id, calobj, NULL);
 
 	e_cal_component_free_id (id);
@@ -239,7 +251,7 @@ retrieval_done (SoupMessage *msg, ECalBackendHttp *cbhttp)
 		if (newuri) {
 			g_free (priv->uri);
 
-			priv->uri = webcal_to_http_method (newuri);
+			priv->uri = webcal_to_http_method (newuri, FALSE);
 			begin_retrieval_cb (cbhttp);
 		} else {
 			if (!priv->opened) {
@@ -424,8 +436,13 @@ begin_retrieval_cb (ECalBackendHttp *cbhttp)
 		g_object_unref (conf_client);
 	}
 
-	if (priv->uri == NULL)
-		priv->uri = webcal_to_http_method (e_cal_backend_get_uri (E_CAL_BACKEND (cbhttp)));
+	if (priv->uri == NULL) {
+		ESource *source = e_cal_backend_get_source (E_CAL_BACKEND (cbhttp));
+		const char *secure_prop = e_source_get_property (source, "secure");
+		
+		priv->uri = webcal_to_http_method (e_cal_backend_get_uri (E_CAL_BACKEND (cbhttp)),
+		                                   (secure_prop && g_str_equal(e_source_get_property (source, "secure"), "1")));
+	}
 
 	/* create message to be sent to server */
 	soup_message = soup_message_new (SOUP_METHOD_GET, priv->uri);
@@ -504,9 +521,11 @@ e_cal_backend_http_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_
 			return GNOME_Evolution_Calendar_OtherError;	
 		}
 		
-		e_cal_backend_cache_put_default_timezone (priv->cache, priv->default_zone);
-		e_cal_backend_http_add_timezone (backend, cal, (const char *) icaltimezone_get_tzid (priv->default_zone));
-	
+		if (priv->default_zone) {
+			e_cal_backend_cache_put_default_timezone (priv->cache, priv->default_zone);
+			e_cal_backend_http_add_timezone (backend, cal, (const char *) icaltimezone_get_tzid (priv->default_zone));
+		}
+
 		if (priv->mode == CAL_MODE_LOCAL) 
 			return GNOME_Evolution_Calendar_Success;
 		
@@ -638,7 +657,7 @@ e_cal_backend_http_get_object (ECalBackendSync *backend, EDataCal *cal, const ch
 {
 	ECalBackendHttp *cbhttp;
 	ECalBackendHttpPrivate *priv;
-	ECalComponent *comp;
+	ECalComponent *comp = NULL;
 
 	cbhttp = E_CAL_BACKEND_HTTP (backend);
 	priv = cbhttp->priv;
@@ -653,7 +672,7 @@ e_cal_backend_http_get_object (ECalBackendSync *backend, EDataCal *cal, const ch
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 
 	*object = e_cal_component_get_as_string (comp);
-	g_free (comp);
+	g_object_unref (comp);
 
 	return GNOME_Evolution_Calendar_Success;
 }
