@@ -7,15 +7,15 @@
  *    	    Michael Zucchi <notzed@ximian.com>
  *
  * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU General Public 
+ * modify it under the terms of version 2 of the GNU Lesser General Public 
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -30,7 +30,6 @@
 #include <string.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <dirent.h>
 #include <errno.h>
 
 #include <camel/camel-url.h>
@@ -174,20 +173,6 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, Cam
 	
 	CAMEL_SERVICE_LOCK(store, connect_lock);
 
-	/* setup store-wide cache */
-	if (store->cache == NULL) {
-		if (store->storage_path == NULL)
-			goto fail;
-		
-		store->cache = camel_data_cache_new (store->storage_path, 0, ex);
-		if (store->cache == NULL)
-			goto fail;
-		
-		/* Default cache expiry - 2 weeks old, or not visited in 5 days */
-		camel_data_cache_set_expire_age (store->cache, 60*60*24*14);
-		camel_data_cache_set_expire_access (store->cache, 60*60*24*5);
-	}
-	
 	if (ssl_mode != MODE_CLEAR) {
 #ifdef HAVE_SSL
 		if (ssl_mode == MODE_TLS) {
@@ -254,6 +239,7 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, Cam
 
 	/* if we have username, try it here */
 	if (service->url->user != NULL
+	    && service->url->user[0]
 	    && camel_nntp_try_authenticate(store, ex) != NNTP_AUTH_ACCEPTED)
 		goto fail;
   	
@@ -497,6 +483,7 @@ nntp_folder_info_from_store_info (CamelNNTPStore *store, gboolean short_notation
 	
 	fi->unread = si->unread;
 	fi->total = si->total;
+	fi->flags = si->flags;
 	path = alloca(strlen(fi->full_name)+2);
 	sprintf(path, "/%s", fi->full_name);
 	url = camel_url_new_with_base (base_url, path);
@@ -778,9 +765,10 @@ nntp_store_get_folder_info_all(CamelNNTPStore *nntp_store, const char *top, guin
 			date[6] = ' ';
 			memcpy(date + 7, summary->last_newslist + 8, 6); /* HHMMSS */
 			date[13] = '\0';
-			
-			if (!nntp_get_date (nntp_store, ex))
-				goto error;
+
+			/* Some servers don't support date (!), so fallback if they dont */
+			if (!nntp_get_date (nntp_store, NULL))
+				goto do_complete_list_nodate;
 			
 			ret = camel_nntp_command (nntp_store, ex, NULL, (char **) &line, "newgroups %s", date);
 			if (ret == -1)
@@ -800,9 +788,8 @@ nntp_store_get_folder_info_all(CamelNNTPStore *nntp_store, const char *top, guin
 		do_complete_list:
 			/* seems we do need a complete list */
 			/* at first, we do a DATE to find out the last load occasion */
-			if (!nntp_get_date (nntp_store, ex))
-				goto error;
-			
+			nntp_get_date (nntp_store, NULL);
+		do_complete_list_nodate:
 			ret = camel_nntp_command (nntp_store, ex, NULL, (char **)&line, "list");
 			if (ret == -1)
 				goto error;
@@ -1010,6 +997,9 @@ nntp_store_finalize (CamelObject *object)
 		g_free(xover);
 		xover = xn;
 	}
+
+	if (nntp_store->cache)
+		camel_object_unref(nntp_store->cache);
 	
 	g_free(p);
 }
@@ -1098,8 +1088,16 @@ nntp_construct (CamelService *service, CamelSession *session,
 		nntp_store->folder_hierarchy_relative = TRUE;
 	else
 		nntp_store->folder_hierarchy_relative = FALSE;
-}
 
+	/* setup store-wide cache */
+	nntp_store->cache = camel_data_cache_new(nntp_store->storage_path, 0, ex);
+	if (nntp_store->cache == NULL)
+		return;
+		
+	/* Default cache expiry - 2 weeks old, or not visited in 5 days */
+	camel_data_cache_set_expire_age(nntp_store->cache, 60*60*24*14);
+	camel_data_cache_set_expire_access(nntp_store->cache, 60*60*24*5);
+}
 
 static void
 nntp_store_init (gpointer object, gpointer klass)
