@@ -37,6 +37,8 @@
 #include <libedataserverui/e-contact-store.h>
 #include <libedataserverui/e-book-auth-util.h>
 #include <libedataserver/e-sexp.h>
+#include <libedataserver/e-categories.h>
+#include <libedataserver/libedataserver-private.h>
 #include "e-name-selector-dialog.h"
 
 typedef struct {
@@ -93,6 +95,43 @@ e_name_selector_dialog_set_property (GObject *object, guint prop_id,
 {
 }
 
+/* FIXME: category_list should become part of ENameSelectorDialog structure */
+GList *category_list;
+
+static int
+compare_func (const void *voida, const void *voidb)
+{
+	const char *stringa = voida;
+        const char *stringb = voidb;
+	return g_utf8_collate (stringa, stringb);
+}
+
+static void
+e_name_selector_dialog_populate_categories (ENameSelectorDialog *name_selector_dialog)
+{
+	GtkWidget *category_option_menu;
+	GtkWidget *category_menu;
+	GList *l;
+	category_option_menu = glade_xml_get_widget (name_selector_dialog->gui, "optionmenu-category");
+
+	category_list = e_categories_get_list () ;
+	category_list = g_list_prepend (category_list, "Any Category");
+	g_list_sort (category_list, compare_func);
+
+	category_menu = gtk_menu_new ();
+	l = category_list;
+	while (l) {	
+		GtkWidget *item;
+		item = gtk_menu_item_new_with_label (l->data);
+		gtk_menu_shell_append (GTK_MENU_SHELL (category_menu), item);
+		l = l->next;
+	}
+	gtk_widget_show_all (category_menu);
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (category_option_menu), category_menu);
+	
+	g_signal_connect_swapped (category_option_menu, "changed", G_CALLBACK (search_changed), name_selector_dialog);
+}
+
 static void
 e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 {
@@ -102,11 +141,16 @@ e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 	GtkWidget         *widget;
 	GtkWidget         *container;
 	GtkWidget	  *label;
+	GtkTreeSelection  *selection;
 	ESourceList       *source_list;
+	char              *gladefile;
 
 	/* Get Glade GUI */
-
-	name_selector_dialog->gui = glade_xml_new (E_DATA_SERVER_UI_GLADEDIR "/e-name-selector-dialog.glade", NULL, GETTEXT_PACKAGE);
+	gladefile = g_build_filename (E_DATA_SERVER_UI_GLADEDIR,
+				      "e-name-selector-dialog.glade",
+				      NULL);
+	name_selector_dialog->gui = glade_xml_new (gladefile, NULL, GETTEXT_PACKAGE);
+	g_free (gladefile);
 
 	widget = glade_xml_get_widget (name_selector_dialog->gui, "name-selector-box");
 	if (!widget) {
@@ -154,6 +198,9 @@ e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 	gtk_tree_view_column_set_cell_data_func (column, cell_renderer,
 						 (GtkTreeCellDataFunc) contact_column_formatter,
 						 name_selector_dialog, NULL);
+
+	selection = gtk_tree_view_get_selection (name_selector_dialog->contact_view);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 	gtk_tree_view_append_column (name_selector_dialog->contact_view, column);
 	g_signal_connect_swapped (name_selector_dialog->contact_view, "row-activated",
 				  G_CALLBACK (contact_activated), name_selector_dialog);
@@ -175,6 +222,29 @@ e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 	/* Create source menu */
 
 	widget = e_source_option_menu_new (name_selector_dialog->source_list);
+        
+	GConfClient *gconf_client;	
+	char *uid;
+	
+	gconf_client = gconf_client_get_default();
+	uid = gconf_client_get_string (gconf_client, "/apps/evolution/addressbook/display/primary_addressbook",
+			NULL);
+	g_object_unref (gconf_client);
+	if (uid) {
+		ESource *source = e_source_list_peek_source_by_uid(name_selector_dialog->source_list, uid);
+		if (source) {
+			e_source_option_menu_select ((ESourceOptionMenu *)widget, source);
+			source_selected (name_selector_dialog, source);
+		}
+		else {
+			source_selected (name_selector_dialog, find_first_source (name_selector_dialog->source_list));
+		}
+		g_free (uid);
+	}
+	else {
+		source_selected (name_selector_dialog, find_first_source (name_selector_dialog->source_list));
+	}
+
 	g_signal_connect_swapped (widget, "source_selected", G_CALLBACK (source_selected), name_selector_dialog);
 
 	label = glade_xml_get_widget (name_selector_dialog->gui, "AddressBookLabel");
@@ -185,6 +255,8 @@ e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 	container = glade_xml_get_widget (name_selector_dialog->gui, "source-menu-box");
 	gtk_box_pack_start (GTK_BOX (container), widget, TRUE, TRUE, 0);
 
+	e_name_selector_dialog_populate_categories (name_selector_dialog);
+
 	/* Set up search-as-you-type signal */
 
 	widget = glade_xml_get_widget (name_selector_dialog->gui, "search");
@@ -194,7 +266,6 @@ e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 
 	/* TODO: Remember last used source */
 
-	source_selected (name_selector_dialog, find_first_source (name_selector_dialog->source_list));
 
 	/* Set up dialog defaults */
 
@@ -209,8 +280,9 @@ e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 	gtk_dialog_set_has_separator    (GTK_DIALOG (name_selector_dialog), FALSE);
 	gtk_container_set_border_width  (GTK_CONTAINER (name_selector_dialog), 4);
 	gtk_window_set_title            (GTK_WINDOW (name_selector_dialog), _("Select Contacts from Address Book"));
+	gtk_widget_grab_focus (widget);
 }
-
+	
 /* Partial, repeatable destruction. Release references. */
 static void
 e_name_selector_dialog_dispose (GObject *object)
@@ -222,7 +294,7 @@ e_name_selector_dialog_dispose (GObject *object)
 					      name_selector_dialog);
 
 	remove_books (name_selector_dialog);
-
+	
 	if (G_OBJECT_CLASS (e_name_selector_dialog_parent_class)->dispose)
 		G_OBJECT_CLASS (e_name_selector_dialog_parent_class)->dispose (object);
 }
@@ -232,6 +304,7 @@ static void
 e_name_selector_dialog_finalize (GObject *object)
 {
 	/* TODO: Free stuff */
+	g_list_free (category_list);
 
 	if (G_OBJECT_CLASS (e_name_selector_dialog_parent_class)->finalize)
 		G_OBJECT_CLASS (e_name_selector_dialog_parent_class)->finalize (object);
@@ -415,10 +488,11 @@ static void
 selection_changed (GtkTreeSelection *selection, SelData *data)
 {
 	GtkTreeSelection *contact_selection;
-	gboolean          have_selection;
-		
+	gboolean          have_selection = FALSE;
+
 	contact_selection = gtk_tree_view_get_selection (data->view);
-	have_selection = gtk_tree_selection_get_selected (contact_selection, NULL, NULL);
+	if (gtk_tree_selection_count_selected_rows (contact_selection) > 0)
+		have_selection = TRUE;
 	gtk_widget_set_sensitive (GTK_WIDGET (data->button), have_selection);
 }
 
@@ -549,6 +623,8 @@ add_section (ENameSelectorDialog *name_selector_dialog,
 	data->button = section.remove_button;
 	g_object_set_data_full ((GObject *)section.destination_view, "sel-change-data", data, g_free);
 	selection = gtk_tree_view_get_selection(section.destination_view);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
+
 	g_signal_connect(selection, "changed",
 				  G_CALLBACK (selection_changed), data);
 
@@ -613,20 +689,41 @@ model_section_removed (ENameSelectorDialog *name_selector_dialog, const gchar *n
  * Addressbook selector *
  * -------------------- */
 
+static void
+status_message(EBookView *view, const gchar *message, ENameSelectorDialog *dialog) 
+{
+	if(message == NULL)
+		gtk_label_set_text(dialog->status_label, "");
+	else
+		gtk_label_set_text(dialog->status_label, message);
+}
+
+static void
+sequence_complete(EBookView *view, EBookViewStatus status, ENameSelectorDialog *dialog)
+{
+	status_message(view, NULL, dialog);
+}
+
 static void
 book_opened (EBook *book, EBookStatus status, gpointer data)
 {
 	ENameSelectorDialog *name_selector_dialog = E_NAME_SELECTOR_DIALOG (data);
 	EContactStore       *contact_store;
+	EBookView           *view; 
 
 	if (status != E_BOOK_ERROR_OK) {
 		/* TODO: Handle errors gracefully */
+		gtk_label_set_text(name_selector_dialog->status_label, "Error loading addressbook");
 		g_warning ("ENameSelectorDialog failed to open book!");
 		return;
 	}
 
 	contact_store = e_name_selector_model_peek_contact_store (name_selector_dialog->name_selector_model);
 	e_contact_store_add_book (contact_store, book);
+	view = find_contact_source_by_book_return_view(contact_store, book);
+	g_signal_connect(view, "status_message", G_CALLBACK(status_message), name_selector_dialog);
+	g_signal_connect(view, "sequence_complete", G_CALLBACK(sequence_complete), name_selector_dialog);
+
 	g_object_unref (book);
 	name_selector_dialog->pending_book = NULL;
 }
@@ -651,20 +748,45 @@ search_changed (ENameSelectorDialog *name_selector_dialog)
 {
 	EContactStore *contact_store;
 	EBookQuery    *book_query;
+	GtkWidget     *category_option_menu;
 	const gchar   *text;
+	gint 	      category_id;	
 	gchar         *text_escaped;
 	gchar         *query_string;
+	const gchar   *category;
+	gchar         *category_escaped;
+
+	category_option_menu = glade_xml_get_widget(name_selector_dialog->gui, "optionmenu-category");
+	category_id = gtk_option_menu_get_history (GTK_OPTION_MENU(category_option_menu));
+	category = g_list_nth_data (category_list, category_id);
+	category_escaped = escape_sexp_string (category);
+
+	if (!category)
+		return;
 
 	text = gtk_entry_get_text (name_selector_dialog->search_entry);
 	text_escaped = escape_sexp_string (text);
-	query_string = g_strdup_printf ("(or (beginswith \"file_as\" %s) "
-					"    (beginswith \"full_name\" %s) "
-					"    (beginswith \"email\" %s) "
-					"    (beginswith \"nickname\" %s))",
-					text_escaped, text_escaped, text_escaped, text_escaped);
+
+	if ( !strcmp (category,"Any Category")) {
+		query_string = g_strdup_printf ("(or (beginswith \"file_as\" %s) "
+						"    (beginswith \"full_name\" %s) "
+						"    (beginswith \"email\" %s) "
+						"    (beginswith \"nickname\" %s)))",
+						text_escaped, text_escaped, text_escaped, text_escaped);
+	}
+	else {
+		query_string = g_strdup_printf ("(and (is \"category_list\" %s) "
+						"(or (beginswith \"file_as\" %s) "
+						"    (beginswith \"full_name\" %s) "
+						"    (beginswith \"email\" %s) "
+						"    (beginswith \"nickname\" %s)))",
+						category_escaped,text_escaped, text_escaped, text_escaped, text_escaped);
+	}
+
 	book_query = e_book_query_from_string (query_string);
 	g_free (query_string);
 	g_free (text_escaped);
+	g_free (category_escaped);
 
 	contact_store = e_name_selector_model_peek_contact_store (name_selector_dialog->name_selector_model);
 	e_contact_store_set_query (contact_store, book_query);
@@ -676,11 +798,12 @@ static void
 contact_selection_changed (ENameSelectorDialog *name_selector_dialog)
 {
 	GtkTreeSelection *contact_selection;
-	gboolean          have_selection;
+	gboolean          have_selection = FALSE;
 	gint              i;
-
+	
 	contact_selection = gtk_tree_view_get_selection (name_selector_dialog->contact_view);
-	have_selection = gtk_tree_selection_get_selected (contact_selection, NULL, NULL);
+	if (gtk_tree_selection_count_selected_rows (contact_selection))
+		have_selection = TRUE;
 
 	for (i = 0; i < name_selector_dialog->sections->len; i++) {
 		Section *section = &g_array_index (name_selector_dialog->sections, Section, i);
@@ -694,7 +817,7 @@ contact_activated (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path)
 	EContactStore     *contact_store;
 	EDestinationStore *destination_store;
 	EContact          *contact;
-	GtkTreeIter        iter;
+	GtkTreeIter       iter;
 	Section           *section;
 	gint               email_n;
 
@@ -703,8 +826,7 @@ contact_activated (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path)
 	contact_store = e_name_selector_model_peek_contact_store (name_selector_dialog->name_selector_model);
 
 	/* If we have no sections, we can't transfer */
-
-	if (name_selector_dialog->sections->len == 0)
+	if (name_selector_dialog->sections->len == 0) 
 		return;
 
 	/* Get the contact to be transferred */
@@ -721,6 +843,7 @@ contact_activated (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path)
 		return;
 	}
 
+
 	section = &g_array_index (name_selector_dialog->sections, Section, 0);
 	if (!e_name_selector_model_peek_section (name_selector_dialog->name_selector_model,
 						 section->name, NULL, &destination_store)) {
@@ -772,7 +895,8 @@ remove_selection (ENameSelectorDialog *name_selector_dialog, GtkTreeView *tree_v
 	EDestinationStore *destination_store;
 	EDestination      *destination;
 	Section           *section;
-	GtkTreeIter        iter;
+	GtkTreeSelection  *selection;
+	GList		  *rows, *l;
 
 	section_index = find_section_by_tree_view (name_selector_dialog, tree_view);
 	if (section_index < 0) {
@@ -787,13 +911,31 @@ remove_selection (ENameSelectorDialog *name_selector_dialog, GtkTreeView *tree_v
 		return FALSE;
 	}
 
-	if (!gtk_tree_selection_get_selected (gtk_tree_view_get_selection (tree_view), NULL, &iter))
+	selection = gtk_tree_view_get_selection (tree_view);
+	if (!gtk_tree_selection_count_selected_rows (selection)) {
+		g_warning ("ENameSelectorDialog remove button clicked, but no selection!");
 		return FALSE;
+	}
 
-	destination = e_destination_store_get_destination (destination_store, &iter);
-	g_assert (destination);
+	rows = gtk_tree_selection_get_selected_rows (selection, NULL);
+	rows = g_list_reverse (rows);
 
-	e_destination_store_remove_destination (destination_store, destination);
+	for (l = rows; l; l = g_list_next(l)) {
+		GtkTreeIter iter;
+		GtkTreePath *path = l->data;
+
+		if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (destination_store),
+					      &iter, path))
+			g_assert_not_reached ();
+
+		gtk_tree_path_free (path);
+
+		destination = e_destination_store_get_destination (destination_store, &iter);
+		g_assert (destination);
+
+		e_destination_store_remove_destination (destination_store, destination);
+	}
+	g_list_free (rows);
 
 	return TRUE;
 }
@@ -827,32 +969,23 @@ transfer_button_clicked (ENameSelectorDialog *name_selector_dialog, GtkButton *t
 	EContactStore     *contact_store;
 	EDestinationStore *destination_store;
 	GtkTreeSelection  *selection;
-	GtkTreeIter        iter;
 	EContact          *contact;
 	gint               section_index;
 	Section           *section;
 	gint               email_n;
+	GList		  *rows, *l;
 
 	/* Get the contact to be transferred */
 
 	contact_store = e_name_selector_model_peek_contact_store (name_selector_dialog->name_selector_model);
 	selection = gtk_tree_view_get_selection (name_selector_dialog->contact_view);
 
-	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+	if (!gtk_tree_selection_count_selected_rows (selection)) {
 		g_warning ("ENameSelectorDialog transfer button clicked, but no selection!");
 		return;
 	}
 
-	sort_iter_to_contact_store_iter (name_selector_dialog, &iter, &email_n);
-
-	contact = e_contact_store_get_contact (contact_store, &iter);
-	if (!contact) {
-		g_warning ("ENameSelectorDialog could not get selected contact!");
-		return;
-	}
-
 	/* Get the target section */
-
 	section_index = find_section_by_transfer_button (name_selector_dialog, transfer_button);
 	if (section_index < 0) {
 		g_warning ("ENameSelectorDialog got click from unknown button!");
@@ -866,7 +999,32 @@ transfer_button_clicked (ENameSelectorDialog *name_selector_dialog, GtkButton *t
 		return;
 	}
 
-	add_destination (destination_store, contact, email_n);
+	rows = gtk_tree_selection_get_selected_rows (selection, NULL);
+	rows = g_list_reverse (rows);
+
+	for (l = rows; l; l = g_list_next(l)) {
+		GtkTreeIter iter;
+		GtkTreePath *path = l->data;
+
+		if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (name_selector_dialog->contact_sort),
+				      &iter, path)) {
+			gtk_tree_path_free (path);
+			return ;
+		}
+			
+		gtk_tree_path_free (path);
+		sort_iter_to_contact_store_iter (name_selector_dialog, &iter, &email_n);
+
+		contact = e_contact_store_get_contact (contact_store, &iter);
+		if (!contact) {
+			g_warning ("ENameSelectorDialog could not get selected contact!");
+			g_list_free (rows);
+			return;
+		}
+
+		add_destination (destination_store, contact, email_n);
+	}
+	g_list_free (rows);
 }
 
 /* --------------------- *
@@ -928,8 +1086,11 @@ setup_name_selector_model (ENameSelectorDialog *name_selector_dialog)
 
 	name_selector_dialog->contact_sort = GTK_TREE_MODEL_SORT (
 		gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (contact_filter)));
+
+	/* sort on full name as we display full name in name selector dialog */
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (name_selector_dialog->contact_sort),
-					      E_CONTACT_FILE_AS, GTK_SORT_ASCENDING);
+					      E_CONTACT_FULL_NAME, GTK_SORT_ASCENDING);
+
 	gtk_tree_view_set_model (name_selector_dialog->contact_view,
 				 GTK_TREE_MODEL (name_selector_dialog->contact_sort));
 
